﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace HornerovaSchemaGui
{
    public partial class Form1 : Form
    {
        #region premmene
        /// <summary>
        /// Rad polynomu
        /// </summary>
        private int degree = -1;
        /// <summary>
        /// Nacitane koeficienty
        /// </summary>
        private int[] coefficients;
        /// <summary>
        /// Horner
        /// </summary>
        private Horner myHorner = new Horner();
        #endregion

        /// <summary>
        /// Hlavne okno
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            this.Text = "Hornerova schema GUI";
            this.ShowAuthor();
        }
        /// <summary>
        /// Vypocet koeficientov
        /// </summary>
        private void calculate()
        {
            this.coefficients = Utils.GetCoefficients(this.textBox1.Text);
            this.degree = Utils.GetDegree(this.coefficients);

            if (this.degree >= 0)
            {
                this.label1.Text = Utils.GetPolynomialFromCoefficients(this.coefficients);
                this.groupBox1.Text = Utils.DegreeToString(this.degree);

                this.myHorner.Calculate(coefficients);
                var result = myHorner.GetResult();
                var roots = result.Select(r => r.Root).ToArray();
                if (roots.Length == 0)
                {
                    this.ShowError();
                }
                this.label3.Text = Utils.GetPolynomialFromRoots(roots);
                this.label2.Text = Utils.RootsToString(roots);

                this.addColumns();

                foreach (var row1 in result)
                {
                    this.addRow(row1);
                }
            }
        }
        
        /// <summary>
        /// Pridanie polynomu v tvare :
        /// f(x) = a4x^4 + a3x^3 + a2x^2 +  a1x^1 + a0x^0 
        /// 5 	        4 	            3 	            2 	            1 	            0
        /// b5 = a5     b4 = a4 + x0b5  b3 = a3 + x0b4  b2 = a2 + x0b3  b1 = a1 + x0b2  b0 = a0 + x0b1
        /// </summary>
        private void addColumns()
        {           
            var row1 = new string[this.degree + 2];
            row1[0] = "";

            this.listView1.Clear();
            this.listView1.Columns.Add("K", 30);

            for (int i = this.degree, j = 1; i >= 0; i--, j++)
            {
                this.listView1.Columns.Add(i.ToString(), 100);
                if (i == this.degree)
                {
                    row1[j] = string.Format("b{0}=a{1}", i, i);
                }
                else
                {
                    row1[j] = string.Format("b{0}=a{1} + x0b{2}", i, i, i + 1);
                }
            }

            var itm = new ListViewItem(row1);
            this.listView1.Items.Add(itm);
        }
        /// <summary>
        /// Zobrazenie hodnot pre kazdy koren polynomu
        /// </summary>
        /// <param name="result"></param>
        private void addRow(Result result)
        {
            var itm1 = new ListViewItem(Utils.ResultToStringArray(result));
            this.listView1.Items.Add(itm1);
        }
        #region ButtonAction
        /// <summary>
        /// Tlacidlo Calculate
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            this.calculate();           
        }
        /// <summary>
        /// Tlacidlo Exit
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Tlacidlo Author
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void authorToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.ShowAuthor();
        }
        /// <summary>
        /// Tlacidlo Help
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void theoryToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start("http://en.wikipedia.org/wiki/Horner%27s_method");
        }
        #endregion
        #region MessageBox
        /// <summary>
        /// Zobrazenie autora
        /// </summary>
        private void ShowAuthor()
        {
            MessageBox.Show("Johny Foxter\n Toto je len demo.", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        /// <summary>
        /// Zobrazenie chyby
        /// </summary>
        private void ShowError()
        {
            MessageBox.Show("Neboli najdene korene polynomu v mnozine celych cisel", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #endregion

    }
}
