﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HornerovaSchemaGui
{
    public class Horner
    {
        /// <summary>
        /// Ulozene vysledky vypoctu
        /// </summary>
        private List<Result> results;
        /// <summary>
        /// Horner
        /// </summary>
        public Horner()
        {
            this.results = new List<Result>();
        }
        /// <summary>
        /// Ziskanie vysledkov vypoctu
        /// </summary>
        /// <returns></returns>
        public List<Result> GetResult()
        {
            return this.results;
        }
        /// <summary>
        /// Vypocet vsetkych delitelov cisla
        /// </summary>
        /// <param name="n">cislo - 21</param>
        /// <returns>pole cisel [1,3,7,21]</returns>
        private IEnumerable<int> getDivisors(int n)
        {
            if (n == 0)
                return (IEnumerable<int>)new int[] { 0 };
            else
                return Enumerable.Range(-Math.Abs(n), 2 * Math.Abs(n) + 1)
                    .Where(a => a != 0)
                    .Where(a => (n % a) == 0);
        }
        /// <summary>
        /// Overi ci je cislo x korenom polynomu
        /// </summary>
        /// <param name="coefficientsA">vstupne koefic.</param>
        /// <param name="x">mozny koren polynomu</param>
        /// <param name="coefficientsB">ak je x korenom,tak vrati dalsie koef na dalsi vypocet.</param>
        /// <returns>True/False - ak je korenom polynomu,vrati true</returns>
        private bool findRootWithHornerScheme(int[] coefficientsA, int x, out int[] coefficientsB)
        {
            var lenght = coefficientsA.Length;
            var tmpB = new int[lenght];

            coefficientsB = new int[lenght - 1];
            tmpB[0] = coefficientsA[0];

            for (int i = 1; i < lenght; i++)
            {
                tmpB[i] = tmpB[i - 1] * x + coefficientsA[i];
            }

            //ak je posledny koefiecient B == 0 ,tak zadane x je korenom polynomu
            if (tmpB[lenght - 1] == 0)
            {
                Array.Copy(tmpB, coefficientsB, lenght - 1);
                return true;//bol najdeny koren
            }
            //nebol najdeny koren a metoda vrati false
            return false;
        }
        /// <summary>
        /// Recurzivna methoda - Spravi vypocet pre vsetky mozne korene polynomu
        /// </summary>
        /// <param name="coefficientsA">vstupne koef.</param>
        private void hornerCalculate(int[] coefficientsA)
        {
            if (coefficientsA.Length > 1)
            {
                int[] tmpCoefficients1, tmpCoefficients2;

                int[] posiblesRoots = this.getDivisors(coefficientsA[coefficientsA.Length - 1]).ToArray();

                tmpCoefficients1 = coefficientsA;

                foreach (var root in posiblesRoots)
                {
                    if (findRootWithHornerScheme(tmpCoefficients1, root, out tmpCoefficients2))
                    {
                        var result = new Result() { Root = root, Coefficients = tmpCoefficients2 };
                        this.results.Add(result);

                        tmpCoefficients1 = tmpCoefficients2;
                        this.hornerCalculate(tmpCoefficients1);
                        break;
                    }
                }
            }

        }
        /// <summary>
        /// Hlavna methoda na vypocet korenov polynomu
        /// </summary>
        /// <param name="coefficientsA">vstupne koef.</param>
        public void Calculate(int[] coefficientsA)
        {
            this.results.Clear();
            this.hornerCalculate(coefficientsA);
        }
    }
}
