﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HornerovaSchemaGui
{
    /// <summary>
    /// Pomocna trieda
    /// </summary>
    public class Utils
    {
        /// <summary>
        /// Metoda , ktora rozlozi vstupny text na pole koeficientov
        /// </summary>
        /// <param name="text">Text v tvare : 1;2;3;4;</param>
        /// <returns>pole koeficientov</returns>
        public static int[] GetCoefficients(string text)
        {
            int n;
            return text.Split(new string[] { ",", ";" }, StringSplitOptions.RemoveEmptyEntries)
                 .Select(s => int.TryParse(s, out n) ? n : 0)
                 .ToArray(); ;
        }
        /// <summary>
        /// Zisti rad polynomu na zaklade zadanych koeficientov
        /// </summary>
        /// <param name="coefficients">pole koeficientov</param>
        /// <returns>Rad polynomu</returns>
        public static int GetDegree(int[] coefficients)
        {
            return coefficients.Length - 1;
        }
        /// <summary>
        /// Zobrazenie funkcie polynomu na zaklade zadanych koeficientov
        /// </summary>
        /// <param name="coefficients">pole koeficientov</param>
        /// <returns>string - f(x) = a4x^4 + a3x^3 + a2x^2 +  a1x^1 + a0x^0 </returns>
        public static string GetPolynomialFromCoefficients(int[] coefficients)
        {
            var fnc = "f(x) = ";
            var degree = GetDegree(coefficients);

            for (int i = 0; i <= degree; i++)
            {
                if (coefficients[i] != 0)
                {
                    if (i < degree - 1)
                    {
                        fnc += string.Format("({0})x^({1})", coefficients[i], degree - i);
                        fnc += coefficients[i + 1] != 0 ? " + " : "";
                    }
                    else if (i == degree - 1)
                    {
                        fnc += string.Format("({0})x", coefficients[i]);
                        fnc += coefficients[i + 1] != 0 ? " + " : "";
                    }
                    else
                    {
                        fnc += string.Format("({0})", coefficients[i]);
                    }
                }
            }

            return fnc;
        }
        /// <summary>
        /// Zobrazenie funkcie polynomu
        /// </summary>
        /// <param name="roots">korene polynomu</param>
        /// <returns>string - f(x) = (x-c)(x-d)</returns>
        public static string GetPolynomialFromRoots(int[] roots)
        {
            var fnc = "f(x) = ";

            for (int i = 0; i < roots.Length; i++)
            {
                if (roots[i] > 0)
                    fnc += string.Format("( x - {0} )", roots[i]);
                else if (roots[i] == 0)
                    fnc += "( x )";
                else
                    fnc += string.Format("( x + {0} )", roots[i] * (-1));
            }

            return fnc;
        }
        /// <summary>
        /// Spojenie korenov do jedneho stringu
        /// </summary>
        /// <param name="roots"></param>
        /// <returns></returns>
        public static string RootsToString(int[] roots)
        {
            return String.Join(";", roots);
        }
        /// <summary>
        /// Prevede int[] to string[]
        /// </summary>
        /// <param name="coefficients"></param>
        /// <returns></returns>
        public static string[] IntToStringArray(int[] coefficients)
        {
            return coefficients.Select(c => c.ToString()).ToArray();
        }
        /// <summary>
        /// Prevedie Result to String
        /// </summary>
        /// <param name="result"></param>
        /// <returns></returns>
        public static string[] ResultToStringArray(Result result)
        {
            List<string> r = new List<string>();
            r.Add(result.Root.ToString());
            r.AddRange(IntToStringArray(result.Coefficients));

            return r.ToArray();
        }
        /// <summary>
        /// Vytvorenie nadpisu
        /// </summary>
        /// <param name="degree">Rad polynomu</param>
        /// <returns>string - 4th degree polynomial</returns>
        public static string DegreeToString(int degree)
        {
            string suffix = "";

            if (degree == 1)
                suffix = "st";
            else if (degree == 2)
                suffix = "nd";
            else if (degree == 3)
                suffix = "rd";
            else
                suffix = "th";

            return string.Format("{0}{1} degree polynomial", degree, suffix);
        }

    }
}
